@@ -247,13 +247,13 @@ static struct pri *__pri_new(int fd, int node, int switchtype, struct pri *maste
 	return p;
 }
 
-void pri_call_set_useruser(q931_call *c, char *userchars)
+void pri_call_set_useruser(q931_call *c, const char *userchars)
 {
 	if (userchars)
 		libpri_copy_string(c->useruserinfo, userchars, sizeof(c->useruserinfo));
 }
 
-void pri_sr_set_useruser(struct pri_sr *sr, char *userchars)
+void pri_sr_set_useruser(struct pri_sr *sr, const char *userchars)
 {
 	sr->useruserinfo = userchars;
 }
@@ -471,6 +471,14 @@ int pri_information(struct pri *pri, q931_call *call, char digit)
 	return q931_information(pri, call, digit);
 }
 
+int pri_keypad_facility(struct pri *pri, q931_call *call, char *digits)
+{
+	if (!pri || !call || !digits || !digits[0])
+		return -1;
+
+	return q931_keypad_facility(pri, call, digits);
+}
+
 int pri_notify(struct pri *pri, q931_call *call, int channel, int info)
 {
 	if (!pri || !call)
@@ -521,11 +529,6 @@ int pri_channel_bridge(q931_call *call1, q931_call *call2)
 	if (!call1 || !call2)
 		return -1;
 
-	/* Check switchtype compatibility */
-	if (call1->pri->switchtype != PRI_SWITCH_LUCENT5E ||
-			call2->pri->switchtype != PRI_SWITCH_LUCENT5E)
-		return -1;
-
 	/* Check for bearer capability */
 	if (call1->transcapability != call2->transcapability)
 		return -1;
@@ -535,10 +538,13 @@ int pri_channel_bridge(q931_call *call1, q931_call *call2)
 	if (call1->pri != call2->pri)
 		return -1;
 	
-	if (eect_initiate_transfer(call1->pri, call1, call2))
-		return -1;
+	if (call1->pri->switchtype == PRI_SWITCH_LUCENT5E)
+		return eect_initiate_transfer(call1->pri, call1, call2);
 
-	return 0;
+	if (call1->pri->switchtype == PRI_SWITCH_DMS100)
+		return rlt_initiate_transfer(call1->pri, call1, call2);
+
+	return -1;
 }
 
 int pri_hangup(struct pri *pri, q931_call *call, int cause)
@@ -775,6 +781,7 @@ char *pri_dump_info_str(struct pri *pri)
 	len += sprintf(buf + len, "T203 Timer: %d\n", pri->timers[PRI_TIMER_T203]);
 	len += sprintf(buf + len, "T305 Timer: %d\n", pri->timers[PRI_TIMER_T305]);
 	len += sprintf(buf + len, "T308 Timer: %d\n", pri->timers[PRI_TIMER_T308]);
+	len += sprintf(buf + len, "T309 Timer: %d\n", pri->timers[PRI_TIMER_T309]);
 	len += sprintf(buf + len, "T313 Timer: %d\n", pri->timers[PRI_TIMER_T313]);
 	len += sprintf(buf + len, "N200 Counter: %d\n", pri->timers[PRI_TIMER_N200]);
 
